Package: fabletools
Title: Core Tools for Packages in the 'fable' Framework
Version: 0.7.0
Authors@R: 
    c(person(given = "Mitchell",
             family = "O'Hara-Wild",
             role = c("aut", "cre"),
             email = "mail@mitchelloharawild.com",
             comment = c(ORCID = "0000-0001-6729-7695")),
      person(given = "Rob",
             family = "Hyndman",
             role = "aut"),
      person(given = "Earo",
             family = "Wang",
             role = "aut",
             comment = c(ORCID = "0000-0001-6448-5260")),
      person(given = "Di",
             family = "Cook",
             role = "ctb"),
      person(given = "George",
             family = "Athanasopoulos",
             role = "ctb"),
      person(given = "David",
             family = "Holt",
             role = "ctb"))
Description: Provides tools, helpers and data structures for
    developing models and time series functions for 'fable' and extension
    packages. These tools support a consistent and tidy interface for time
    series modelling and analysis.
License: GPL-3
URL: https://fabletools.tidyverts.org/,
        https://github.com/tidyverts/fabletools
BugReports: https://github.com/tidyverts/fabletools/issues
Depends: R (>= 3.1.3)
Imports: tsibble (>= 0.9.0), tibble (>= 1.4.1), ggplot2 (>= 3.0.0),
        tidyselect, rlang (>= 0.4.5), stats, dplyr (>= 1.0.0), tidyr
        (>= 1.1.0), generics, R6, utils, vctrs (>= 0.2.2),
        distributional (>= 0.3.1), progressr, lifecycle, ggdist,
        scales, cli
Suggests: covr, crayon, fable (>= 0.2.0), future, future.apply, knitr,
        pillar (>= 1.0.1), feasts (>= 0.1.2), rmarkdown, spelling,
        testthat, tsibbledata (>= 0.2.0), lubridate, urca, mvtnorm,
        Matrix, ggtime
VignetteBuilder: knitr
RdMacros: lifecycle
ByteCompile: true
Encoding: UTF-8
Language: en-GB
Config/roxygen2/version: 7.3.3.9000
NeedsCompilation: no
Packaged: 2026-04-29 19:45:30 UTC; mitchell
Author: Mitchell O'Hara-Wild [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6729-7695>),
  Rob Hyndman [aut],
  Earo Wang [aut] (ORCID: <https://orcid.org/0000-0001-6448-5260>),
  Di Cook [ctb],
  George Athanasopoulos [ctb],
  David Holt [ctb]
Maintainer: Mitchell O'Hara-Wild <mail@mitchelloharawild.com>
Repository: CRAN
Date/Publication: 2026-04-30 05:10:17 UTC
Built: R 4.6.0; ; 2026-05-01 21:50:39 UTC; unix
