# Generated by roxygen2: do not edit by hand

S3method("!=",agg_vec)
S3method("$<-",mdl_df)
S3method("==",agg_vec)
S3method("[",dcmp_ts)
S3method("[",fbl_ts)
S3method("[",mdl_df)
S3method("names<-",mdl_df)
S3method(IRF,mdl_df)
S3method(IRF,mdl_ts)
S3method(Ops,lst_mdl)
S3method(Ops,mdl_defn)
S3method(Ops,mdl_lst)
S3method(Ops,mdl_ts)
S3method(accuracy,fbl_ts)
S3method(accuracy,mdl_df)
S3method(accuracy,mdl_ts)
S3method(aggregate_key,tbl_ts)
S3method(as_dable,tbl_df)
S3method(as_dable,tbl_ts)
S3method(as_fable,fbl_ts)
S3method(as_fable,forecast)
S3method(as_fable,grouped_df)
S3method(as_fable,grouped_ts)
S3method(as_fable,tbl_df)
S3method(as_fable,tbl_ts)
S3method(as_mable,data.frame)
S3method(as_tibble,fbl_ts)
S3method(as_tibble,grouped_fbl)
S3method(as_tibble,mdl_df)
S3method(as_tsibble,dcmp_ts)
S3method(as_tsibble,fbl_ts)
S3method(as_tsibble,grouped_fbl)
S3method(augment,lst_mdl)
S3method(augment,mdl_df)
S3method(augment,mdl_lst)
S3method(augment,mdl_ts)
S3method(coef,lst_mdl)
S3method(coef,mdl_df)
S3method(coef,mdl_lst)
S3method(coef,mdl_ts)
S3method(coherent_cmat,default)
S3method(coherent_smat,default)
S3method(coherent_smat,tbl)
S3method(common_periods,default)
S3method(common_periods,interval)
S3method(common_periods,tbl_ts)
S3method(components,mdl_df)
S3method(components,mdl_ts)
S3method(distribution_var,fbl_ts)
S3method(dplyr_col_modify,dcmp_ts)
S3method(dplyr_col_modify,fbl_ts)
S3method(dplyr_col_modify,grouped_fbl)
S3method(dplyr_col_modify,mdl_df)
S3method(dplyr_reconstruct,dcmp_ts)
S3method(dplyr_reconstruct,fbl_ts)
S3method(dplyr_reconstruct,grouped_fbl)
S3method(dplyr_reconstruct,mdl_df)
S3method(dplyr_row_slice,dcmp_ts)
S3method(dplyr_row_slice,fbl_ts)
S3method(dplyr_row_slice,grouped_fbl)
S3method(dplyr_row_slice,mdl_df)
S3method(equation,mdl_df)
S3method(equation,mdl_ts)
S3method(estimate,tbl_ts)
S3method(features,tbl_ts)
S3method(features_all,tbl_ts)
S3method(features_at,tbl_ts)
S3method(features_if,tbl_ts)
S3method(fill_gaps,fbl_ts)
S3method(fitted,"NULL")
S3method(fitted,mdl_df)
S3method(fitted,mdl_ts)
S3method(fitted,model_combination)
S3method(fitted,null_mdl)
S3method(forecast,"NULL")
S3method(forecast,fbl_ts)
S3method(forecast,lst_btmup_mdl)
S3method(forecast,lst_mdl)
S3method(forecast,lst_midout_mdl)
S3method(forecast,lst_mint_mdl)
S3method(forecast,lst_topdwn_mdl)
S3method(forecast,mdl_df)
S3method(forecast,mdl_lst)
S3method(forecast,mdl_ts)
S3method(forecast,model_combination)
S3method(forecast,null_mdl)
S3method(format,agg_vec)
S3method(format,lst_mdl)
S3method(format,mdl_lst)
S3method(format,mdl_ts)
S3method(gather,mdl_df)
S3method(generate,"NULL")
S3method(generate,lst_mdl)
S3method(generate,mdl_df)
S3method(generate,mdl_lst)
S3method(generate,mdl_ts)
S3method(generate,model_combination)
S3method(generate,null_mdl)
S3method(get_frequencies,"NULL")
S3method(get_frequencies,Period)
S3method(get_frequencies,character)
S3method(get_frequencies,numeric)
S3method(glance,lst_mdl)
S3method(glance,mdl_df)
S3method(glance,mdl_lst)
S3method(glance,mdl_ts)
S3method(glance,null_mdl)
S3method(group_by,fbl_ts)
S3method(group_by,grouped_fbl)
S3method(group_data,mdl_df)
S3method(hfitted,mdl_ts)
S3method(hilo,fbl_ts)
S3method(hypothesize,mdl_df)
S3method(hypothesize,mdl_ts)
S3method(interpolate,mdl_df)
S3method(interpolate,mdl_ts)
S3method(invert_transformation,transformation)
S3method(is.na,agg_vec)
S3method(key,mdl_df)
S3method(key_data,mdl_df)
S3method(key_vars,mdl_df)
S3method(mable_vars,mdl_df)
S3method(model,tbl_ts)
S3method(model_sum,decomposition_model)
S3method(model_sum,default)
S3method(model_sum,mdl_ts)
S3method(model_sum,model_combination)
S3method(model_sum,null_mdl)
S3method(outliers,mdl_df)
S3method(outliers,mdl_ts)
S3method(pivot_longer,mdl_df)
S3method(print,transformation)
S3method(rbind,dcmp_ts)
S3method(rbind,fbl_ts)
S3method(recode,agg_vec)
S3method(reconcile,mdl_df)
S3method(refit,"NULL")
S3method(refit,lst_mdl)
S3method(refit,mdl_df)
S3method(refit,mdl_lst)
S3method(refit,mdl_ts)
S3method(refit,null_mdl)
S3method(report,"NULL")
S3method(report,mdl_df)
S3method(report,mdl_ts)
S3method(report,model_combination)
S3method(report,null_mdl)
S3method(residuals,"NULL")
S3method(residuals,mdl_df)
S3method(residuals,mdl_ts)
S3method(residuals,model_combination)
S3method(residuals,null_mdl)
S3method(response,mdl_df)
S3method(response,mdl_ts)
S3method(response_vars,dcmp_ts)
S3method(response_vars,fbl_ts)
S3method(response_vars,mdl_df)
S3method(response_vars,mdl_ts)
S3method(select,fbl_ts)
S3method(select,grouped_fbl)
S3method(select,mdl_df)
S3method(stream,"NULL")
S3method(stream,lst_mdl)
S3method(stream,mdl_df)
S3method(stream,mdl_lst)
S3method(stream,mdl_ts)
S3method(stream,null_mdl)
S3method(summarise,fbl_ts)
S3method(summarise,grouped_fbl)
S3method(tidy,lst_mdl)
S3method(tidy,mdl_df)
S3method(tidy,mdl_lst)
S3method(tidy,mdl_ts)
S3method(tidy,null_mdl)
S3method(transmute,fbl_ts)
S3method(transmute,grouped_fbl)
S3method(transmute,mdl_df)
S3method(ungroup,fbl_ts)
S3method(ungroup,grouped_fbl)
S3method(vec_cast,agg_vec)
S3method(vec_cast,agg_vec.agg_vec)
S3method(vec_cast,agg_vec.character)
S3method(vec_cast,agg_vec.default)
S3method(vec_cast,character.agg_vec)
S3method(vec_cast,character.lst_mdl)
S3method(vec_cast,character.mdl_lst)
S3method(vec_cast,data.frame.dcmp_ts)
S3method(vec_cast,data.frame.fbl_ts)
S3method(vec_cast,data.frame.mdl_df)
S3method(vec_cast,dcmp_ts)
S3method(vec_cast,dcmp_ts.data.frame)
S3method(vec_cast,dcmp_ts.dcmp_ts)
S3method(vec_cast,dcmp_ts.tbl_df)
S3method(vec_cast,fbl_ts)
S3method(vec_cast,fbl_ts.data.frame)
S3method(vec_cast,fbl_ts.fbl_ts)
S3method(vec_cast,fbl_ts.tbl_df)
S3method(vec_cast,mdl_df)
S3method(vec_cast,mdl_df.data.frame)
S3method(vec_cast,mdl_df.mdl_df)
S3method(vec_cast,mdl_df.tbl_df)
S3method(vec_cast,mdl_lst.mdl_lst)
S3method(vec_cast,tbl_df.dcmp_ts)
S3method(vec_cast,tbl_df.fbl_ts)
S3method(vec_cast,tbl_df.mdl_df)
S3method(vec_cast,tbl_ts.dcmp_ts)
S3method(vec_cast,tbl_ts.fbl_ts)
S3method(vec_proxy_compare,agg_vec)
S3method(vec_ptype2,agg_vec)
S3method(vec_ptype2,agg_vec.agg_vec)
S3method(vec_ptype2,agg_vec.character)
S3method(vec_ptype2,agg_vec.default)
S3method(vec_ptype2,character.agg_vec)
S3method(vec_ptype2,data.frame.dcmp_ts)
S3method(vec_ptype2,data.frame.fbl_ts)
S3method(vec_ptype2,data.frame.mdl_df)
S3method(vec_ptype2,dcmp_ts)
S3method(vec_ptype2,dcmp_ts.data.frame)
S3method(vec_ptype2,dcmp_ts.dcmp_ts)
S3method(vec_ptype2,dcmp_ts.tbl_df)
S3method(vec_ptype2,fbl_ts)
S3method(vec_ptype2,fbl_ts.data.frame)
S3method(vec_ptype2,fbl_ts.fbl_ts)
S3method(vec_ptype2,fbl_ts.tbl_df)
S3method(vec_ptype2,mdl_df)
S3method(vec_ptype2,mdl_df.data.frame)
S3method(vec_ptype2,mdl_df.mdl_df)
S3method(vec_ptype2,mdl_df.tbl_df)
S3method(vec_ptype2,mdl_lst.mdl_lst)
S3method(vec_ptype2,tbl_df.dcmp_ts)
S3method(vec_ptype2,tbl_df.fbl_ts)
S3method(vec_ptype2,tbl_df.mdl_df)
S3method(vec_ptype_abbr,agg_vec)
S3method(vec_restore,fbl_ts)
export("%>%")
export(ACF1)
export(CRPS)
export(IRF)
export(MAAPE)
export(MAE)
export(MAPE)
export(MASE)
export(MDA)
export(MDPV)
export(MDV)
export(ME)
export(MPE)
export(MSE)
export(RMSE)
export(RMSSE)
export(accuracy)
export(agg_vec)
export(aggregate_key)
export(as_dable)
export(as_fable)
export(as_mable)
export(as_tsibble)
export(augment)
export(autolayer)
export(autoplot)
export(bias_adjust)
export(bottom_up)
export(box_cox)
export(coherent_cmat)
export(coherent_smat)
export(combination_ensemble)
export(combination_model)
export(combination_weighted)
export(common_periods)
export(common_xregs)
export(components)
export(construct_fc)
export(dable)
export(decomposition_model)
export(directional_accuracy_measures)
export(distribution_accuracy_measures)
export(distribution_var)
export(equation)
export(estimate)
export(fable)
export(feature_set)
export(features)
export(features_all)
export(features_at)
export(features_if)
export(forecast)
export(generate)
export(get_frequencies)
export(glance)
export(hfitted)
export(hilo)
export(hypothesize)
export(interpolate)
export(interval_accuracy_measures)
export(inv_box_cox)
export(invert_transformation)
export(is_aggregated)
export(is_dable)
export(is_fable)
export(is_mable)
export(is_model)
export(is_null_model)
export(mable)
export(mable_vars)
export(middle_out)
export(min_trace)
export(model)
export(model_lhs)
export(model_rhs)
export(model_sum)
export(new_model_class)
export(new_model_definition)
export(new_specials)
export(new_transformation)
export(null_model)
export(outliers)
export(percentile_score)
export(pinball_loss)
export(point_accuracy_measures)
export(quantile_score)
export(reconcile)
export(refit)
export(register_feature)
export(report)
export(response)
export(response_vars)
export(scaled_pinball_loss)
export(scenarios)
export(skill_score)
export(special_xreg)
export(stream)
export(tidy)
export(top_down)
export(unpack_hilo)
export(vars)
export(winkler_score)
import(rlang)
import(tsibble)
import(vctrs)
importFrom(distributional,hilo)
importFrom(dplyr,"%>%")
importFrom(dplyr,anti_join)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,dplyr_col_modify)
importFrom(dplyr,dplyr_reconstruct)
importFrom(dplyr,dplyr_row_slice)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,group_data)
importFrom(dplyr,groups)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,recode)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,semi_join)
importFrom(dplyr,summarise)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(dplyr,vars)
importFrom(generics,accuracy)
importFrom(generics,augment)
importFrom(generics,components)
importFrom(generics,equation)
importFrom(generics,forecast)
importFrom(generics,generate)
importFrom(generics,glance)
importFrom(generics,hypothesize)
importFrom(generics,interpolate)
importFrom(generics,refit)
importFrom(generics,tidy)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,autolayer)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,fortify)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,vars)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,deprecated)
importFrom(stats,fitted)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(tibble,new_tibble)
importFrom(tidyr,gather)
importFrom(tidyr,nest)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,spread)
importFrom(tidyr,unnest)
importFrom(tidyselect,all_of)
importFrom(tsibble,as_tsibble)
importFrom(vctrs,vec_ptype2)
