
<!-- README.md is generated from README.Rmd. Please edit that file -->

# fabletools

<!-- badges: start -->

[![R build
status](https://github.com/tidyverts/fabletools/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/tidyverts/fabletools/actions/workflows/R-CMD-check.yaml)
[![Code test
coverage](https://codecov.io/gh/tidyverts/fabletools/branch/master/graph/badge.svg)](https://app.codecov.io/gh/tidyverts/fabletools?branch=master)
[![lifecycle](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![CRAN
status](https://www.r-pkg.org/badges/version/fabletools)](https://CRAN.R-project.org/package=fabletools)
<!-- badges: end -->

The R package *fabletools* provides tools for building modelling
packages, with a focus on time series forecasting. This package allows
package developers to extend *fable* with additional models, without
needing to depend on the models supported by *fable*.

## Installation

You could install the **stable** version on
[CRAN](https://cran.r-project.org/package=fabletools):

``` r
install.packages("fabletools")
```

You can install the **development** version from
[GitHub](https://github.com/tidyverts/fabletools):

``` r
# install.packages("remotes")
remotes::install_github("tidyverts/fabletools")
```
