#' @importFrom dplyr %>%
#' @export
dplyr::`%>%`

#' @export
tsibble::as_tsibble

#' @importFrom dplyr vars
#' @export
dplyr::vars

#' @importFrom ggplot2 autoplot
#' @export
ggplot2::autoplot

#' @importFrom ggplot2 autolayer
#' @export
ggplot2::autolayer

#' @importFrom generics accuracy
#' @export
generics::accuracy

#' @importFrom generics equation
#' @export
generics::equation

#' @importFrom generics interpolate
#' @export
generics::interpolate

#' @importFrom generics components
#' @export
generics::components

#' @importFrom generics augment
#' @export
generics::augment

#' @importFrom generics glance
#' @export
generics::glance

#' @importFrom generics tidy
#' @export
generics::tidy

#' @importFrom generics hypothesize
#' @export
generics::hypothesize

#' @importFrom generics generate
#' @export
generics::generate

#' @importFrom generics refit
#' @export
generics::refit

#' @importFrom generics forecast
#' @export
generics::forecast

#' @importFrom distributional hilo
#' @export
distributional::hilo
