% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dable.R
\name{is_dable}
\alias{is_dable}
\title{Is the object a dable}
\usage{
is_dable(x)
}
\arguments{
\item{x}{An object.}
}
\description{
Is the object a dable
}
