% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mable.R
\name{mable}
\alias{mable}
\title{Create a new mable}
\usage{
mable(..., key = NULL, model = NULL)
}
\arguments{
\item{...}{A set of name-value pairs.}

\item{key}{Structural variable(s) that identify each model.}

\item{model}{Identifiers for the columns containing model(s).}
}
\description{
A mable (model table) data class (\code{mdl_df}) is a tibble-like data structure
for applying multiple models to a dataset. Each row of the mable refers to a
different time series from the data (identified by the key columns). A mable
must contain at least one column of time series models (\code{mdl_ts}), where the
list column itself (\code{mdl_lst}) describes how these models are related.
}
