% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mable.R
\name{is_mable}
\alias{is_mable}
\title{Is the object a mable}
\usage{
is_mable(x)
}
\arguments{
\item{x}{An object.}
}
\description{
Is the object a mable
}
