% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fable.R
\name{is_fable}
\alias{is_fable}
\title{Is the object a fable}
\usage{
is_fable(x)
}
\arguments{
\item{x}{An object.}
}
\description{
Is the object a fable
}
