% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse.R
\name{parse_model}
\alias{parse_model}
\title{Parse the model specification for specials}
\usage{
parse_model(model)
}
\arguments{
\item{model}{A model definition}
}
\description{
Using a list of defined special functions, the user's formula specification and data
is parsed to extract important modelling components.
}
\keyword{internal}
