% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fabletools-package.R
\docType{package}
\name{fabletools-package}
\alias{fabletools}
\alias{fabletools-package}
\title{fabletools: Core Tools for Packages in the 'fable' Framework}
\description{
Provides tools, helpers and data structures for developing models and time series functions for 'fable' and extension packages. These tools support a consistent and tidy interface for time series modelling and analysis.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://fabletools.tidyverts.org/}
  \item \url{https://github.com/tidyverts/fabletools}
  \item Report bugs at \url{https://github.com/tidyverts/fabletools/issues}
}

}
\author{
\strong{Maintainer}: Mitchell O'Hara-Wild \email{mail@mitchelloharawild.com} (\href{https://orcid.org/0000-0001-6729-7695}{ORCID})

Authors:
\itemize{
  \item Mitchell O'Hara-Wild \email{mail@mitchelloharawild.com} (\href{https://orcid.org/0000-0001-6729-7695}{ORCID})
  \item Rob Hyndman
  \item Earo Wang (\href{https://orcid.org/0000-0001-6448-5260}{ORCID})
}

Other contributors:
\itemize{
  \item Di Cook [contributor]
  \item George Athanasopoulos [contributor]
  \item David Holt [contributor]
}

}
\keyword{package}
