% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{model_rhs}
\alias{model_rhs}
\title{Extract the right hand side of a model}
\usage{
model_rhs(model)
}
\arguments{
\item{model}{A formula}
}
\description{
Extract the right hand side of a model
}
